@@ -3,7 +3,7 @@
 ** distribution information.
 */
 
-/* $Id: acconfig.h,v 1.8 2001/02/09 13:23:35 mrsam Exp $ */
+/* $Id: acconfig.h,v 1.9 2001/03/26 23:22:43 mrsam Exp $ */
 
 /* Whether we must provide a prototype for crypt() */
 #undef	NEED_CRYPT_PROTOTYPE
@@ -28,3 +28,6 @@
 
 /* Whether -lvpopmail has the open_smtp_relay() function */
 #undef HAVE_OPEN_SMTP_RELAY
+
+/* Whether -lvpopmail has the vlogauth() function */
+#undef HAVE_VLOGAUTH
