@@ -19,7 +19,7 @@
 #include	"authmysql.h"
 #include	"authstaticlist.h"
 
-static const char rcsid[]="$Id: authmysql.c,v 1.9 2000/10/14 17:34:44 mrsam Exp $";
+static const char rcsid[]="$Id: authmysql.c,v 1.10 2001/05/20 15:33:00 mrsam Exp $";
 
 static char *auth_mysql_login(const char *service, char *authdata,
 	int issession,
@@ -71,6 +71,7 @@ struct authmysqluserinfo *authinfo;
 	if (callback_func == 0)
 	{
 	static char *maildir=0;
+	static char *quota=0;
 
 		authsuccess(authinfo->home, 0, &authinfo->uid,
 			&authinfo->gid, authinfo->username, "");
@@ -92,6 +93,24 @@ struct authmysqluserinfo *authinfo;
 		{
 			putenv("MAILDIR=");
 		}
+
+		if (authinfo->quota && authinfo->quota[0])
+                {
+                        if (quota)    free(quota);
+                        quota=malloc(sizeof("MAILDIRQUOTA=")+
+                                        strlen(authinfo->quota));
+                        if (!quota)
+                        {
+                                perror("malloc");
+                                exit(1);
+                        }
+                        strcat(strcpy(quota, "MAILDIRQUOTA="), authinfo->quota);
+                        putenv(quota);
+                }
+                else
+                {
+                        putenv("MAILDIRQUOTA=");
+                }
 	}
 	else
 	{
@@ -106,6 +125,8 @@ struct authmysqluserinfo *authinfo;
 		aa.maildir=authinfo->maildir && authinfo->maildir[0] ?
 			authinfo->maildir:0;
 		aa.address=authinfo->username;
+		aa.quota=authinfo->quota && authinfo->quota[0] ?
+			authinfo->quota:0;
 		(*callback_func)(&aa, callback_arg);
 	}
 
