@@ -178,7 +178,7 @@ void auth_mysql_cleanup()
 	}
 }
 
-static struct authmysqluserinfo ui={0, 0, 0, 0, 0};
+static struct authmysqluserinfo ui={0, 0, 0, 0, 0, 0};
 
 struct authmysqluserinfo *auth_mysql_getuserinfo(const char *username)
 {
@@ -189,10 +189,10 @@ MYSQL_ROW	row;
 MYSQL_RES	*result;
 
 const char *crypt_field, *clear_field, *maildir_field, *home_field, 
-           *login_field, *uid_field, *gid_field, *where_clause;
+           *login_field, *uid_field, *gid_field, *quota_field, *where_clause;
 
 static const char query[]=
-	"SELECT %s, %s, %s, %s, %s, %s, %s FROM %s WHERE %s = \"";
+	"SELECT %s, %s, %s, %s, %s, %s, %s, %s FROM %s WHERE %s = \"";
 
 	if (do_connect())	return (0);
 
@@ -206,6 +206,8 @@ static const char query[]=
 		free(ui.home);
 	if (ui.maildir)
 		free(ui.maildir);
+	if (ui.quota)
+		free(ui.quota);
 
 	memset(&ui, 0, sizeof(ui));
 
@@ -247,6 +249,9 @@ static const char query[]=
 	maildir_field=read_env("MYSQL_MAILDIR_FIELD");
 	if (!maildir_field) maildir_field="\"\"";
 
+	quota_field=read_env("MYSQL_QUOTA_FIELD");
+	if (!quota_field) quota_field="\"\""; 
+
 	where_clause=read_env("MYSQL_WHERE_CLAUSE");
 	if (!where_clause) where_clause = "";
 	
@@ -255,7 +260,7 @@ static const char query[]=
 	querybuf=malloc(sizeof(query) + 100 + strlen(user_table) + strlen(defdomain)
 		+ strlen(crypt_field) + strlen(clear_field) + strlen(maildir_field)
 		+ strlen(uid_field) + strlen(gid_field) + 2 * strlen(login_field)
-		+ strlen(home_field) + strlen(where_clause));
+		+ strlen(home_field) + strlen(quota_field) + strlen(where_clause));
 	if (!querybuf)
 	{
 		perror("malloc");
@@ -263,8 +268,8 @@ static const char query[]=
 	}
 
 	sprintf(querybuf, query, login_field, crypt_field, clear_field, 
-		uid_field, gid_field, home_field, maildir_field, user_table,
-		login_field);
+		uid_field, gid_field, home_field, maildir_field, quota_field, 
+		user_table, login_field);
 	p=querybuf+strlen(querybuf);
 	for (strcpy(p, username); *p; p++)
 		if (*p == '"')	*p=' ';	/* No funny business */
@@ -315,8 +320,10 @@ static const char query[]=
 			ui.gid=atol(row[4]);
 			ui.home=strdup(row[5]);
 			ui.maildir=strdup(row[6]);
+			ui.quota=strdup(row[7]);
 			if (!ui.username || !ui.cryptpw ||
-				!ui.home || !ui.maildir)
+				!ui.home || !ui.maildir || 
+				!ui.quota)
 			{
 				mysql_free_result(result);
 				return (0);
