@@ -15,7 +15,7 @@
 #endif
 #include	"auth.h"
 
-static const char rcsid[]="$Id: checkpassword.c,v 1.7 2000/09/03 00:41:09 mrsam Exp $";
+static const char rcsid[]="$Id: checkpassword.c,v 1.8 2001/04/19 01:20:46 mrsam Exp $";
 
 #if HAVE_CRYPT
 #if NEED_CRYPT_PROTOTYPE
@@ -27,6 +27,10 @@ extern char *crypt(const char *, const char *);
 extern int authcheckpasswordmd5(const char *, const char *);
 #endif
 
+#if	HAVE_MD5LIB
+extern int authcheckpasswordsha1(const char *, const char *);
+#endif
+
 int authcheckpassword(const char *password, const char *encrypted_password)
 {
 #if	HAVE_MD5LIB
@@ -36,6 +40,12 @@ int authcheckpassword(const char *password, const char *encrypted_password)
 		return (authcheckpasswordmd5(password, encrypted_password));
 #endif
 
+#if	HAVE_SHA1LIB
+	if (strncasecmp(encrypted_password, "{SHA}", 5) == 0
+		)
+		return (authcheckpasswordsha1(password, encrypted_password));
+#endif
+
 	return (
 #if	HAVE_CRYPT
 		strcmp(encrypted_password,
