@@ -20,7 +20,7 @@
 #include	"authmysql.h"
 
 
-static const char rcsid[]="$Id: preauthmysql.c,v 1.5 2000/07/04 00:08:56 mrsam Exp $";
+static const char rcsid[]="$Id: preauthmysql.c,v 1.6 2001/05/20 15:33:00 mrsam Exp $";
 
 
 int	auth_mysql_pre(const char *user, const char *service,
@@ -48,5 +48,7 @@ struct	authinfo	aa;
 	aa.address=authinfo->username;
 	aa.passwd=authinfo->cryptpw;
 	aa.clearpasswd=authinfo->clearpw;
+	aa.quota=authinfo->quota && authinfo->quota[0] ?
+		authinfo->quota:0;
 	return ((*callback)(&aa, arg));
 }
