@@ -75,7 +75,7 @@
 
 #define	SHARED	"shared"
 
-static const char rcsid[]="$Id: imapd.c,v 1.63 2001/05/10 14:13:46 mrsam Exp $";
+static const char rcsid[]="$Id: imapd.c,v 1.64 2001/06/07 02:49:25 mrsam Exp $";
 
 extern time_t rfc822_parsedt(const char *);
 extern void fetchflags(unsigned long);
@@ -124,24 +124,68 @@ int is_trash(const char *m)
 
 void emptytrash()
 {
-char	*dir;
-const char *p;
-unsigned l;
+	char	*dir, *all_settings, *next_folder, *folder, *p;
+	unsigned l;
 
-	p=getenv("IMAP_EMPTYTRASH");
-	if (!p || !isdigit( (int)(unsigned char)*p))	return;
-	l=atoi(p);
-	if (l > 31)	l=31;
+	all_settings=getenv("IMAP_EMPTYTRASH");
 
-	maildir_getnew( ".", TRASH);
+	if (!all_settings)
+		return;
 
-	dir=maildir_folderdir(".", TRASH);
+	all_settings=strdup(all_settings);
+	if (!all_settings)
+		return;
 
-	if (dir)
+	if (strchr(all_settings, ':') == 0 &&
+	    strchr(all_settings, ',') == 0)
 	{
-		maildir_purge(dir, l * 24 * 60 * 60);
-		free(dir);
+		l=atoi(all_settings);
+
+		if (l <= 0)
+			l=1;
+
+		maildir_getnew(".", TRASH);
+		if ((dir=maildir_folderdir(".", TRASH)))
+		{
+			maildir_purge(dir, l * 24 * 60 * 60);
+			free(dir);
+		}
+		free(all_settings);
+		return;
 	}
+
+	for (folder=all_settings; folder && *folder; )
+	{
+		if (*folder == ',')
+		{
+			++folder;
+			continue;
+		}
+		next_folder=strchr(folder, ',');
+		if (next_folder)
+			*next_folder++=0;
+
+		p=strchr(folder, ':');
+		if (!p)
+		{
+			folder=next_folder;
+			continue;
+		}
+
+		*p++=0;
+
+		l=atoi(p);
+		if (l <= 0)	l=1;
+
+		maildir_getnew(".", folder);
+		if ((dir=maildir_folderdir(".", folder)))
+		{
+			maildir_purge(dir, l * 24 * 60 * 60);
+			free(dir);
+		}
+		folder=next_folder;
+	}
+	free(all_settings);
 }
 
 int is_draft(const char *m)
