@@ -17,7 +17,7 @@
 #include	"imaptoken.h"
 #include	"imapwrite.h"
 
-static const char rcsid[]="$Id: mainloop.c,v 1.4 2001/02/26 05:16:21 mrsam Exp $";
+static const char rcsid[]="$Id: mainloop.c,v 1.5 2001/03/01 02:30:09 mrsam Exp $";
 
 extern int do_imap_command(const char *);
 
@@ -53,6 +53,9 @@ void mainloop(void)
 		{
 		int	rc;
 
+			if (strlen(tag)+strlen(curtoken->tokenbuf) > IT_MAX_ATOM_SIZE)
+				write_error_exit("max atom size too small");
+		  		
 			strncat(tag, curtoken->tokenbuf, IT_MAX_ATOM_SIZE);
 			rc=do_imap_command(tag);
 
