@@ -13,7 +13,7 @@
 #include	<stdlib.h>
 #include	<string.h>
 
-static const char rcsid[]="$Id: msgenvelope.c,v 1.7 2000/10/22 05:23:50 mrsam Exp $";
+static const char rcsid[]="$Id: msgenvelope.c,v 1.8 2001/04/19 23:45:30 mrsam Exp $";
 
 #define	MAX_HEADER_SIZE	8192
 
@@ -172,7 +172,7 @@ char	*q, *r;
 		msgappends(writefunc, q, strlen(q));
 		(*writefunc)("\" \"", 3);
 		if (r)
-			(*writefunc)(r, strlen(r));
+			msgappends(writefunc, r, strlen(r));
 		(*writefunc)("\")", 2);
 		free(q);
 	}
