@@ -2,7 +2,7 @@
 ** Copyright 1998 - 2001 Double Precision, Inc.
 ** See COPYING for distribution information.
 **
-** $Id: pop3dserver.c,v 1.6 2001/02/11 16:44:12 mrsam Exp $
+** $Id: pop3dserver.c,v 1.8 2001/04/13 01:12:33 mrsam Exp $
 */
 
 #if	HAVE_CONFIG_H
@@ -381,7 +381,7 @@ unsigned long *cntr;
 	if (strchr(p, ':') == 0)	strcat(p, ":2,");
 	strcat(p, "S");
 
-	if (lptr	// Don't mark as seen for TOP
+	if (lptr	/* Don't mark as seen for TOP */
 	    || rename(msglist_a[i]->filename, p))
 	{
 		free(p);
@@ -438,7 +438,6 @@ unsigned i;
 
 static void acctout(const char *disc)
 {
-	static const char msg1[]="ERR: ";
 	static const char msg2[]=", user=";
 	static const char msg3[]=", ip=[";
 	static const char msg4[]="], top=";
@@ -453,8 +452,7 @@ static void acctout(const char *disc)
 	p=malloc(strlen(authaddr)+strlen(remoteip)+strlen(disc)+
 		 strlen(num1)+strlen(num2)+200);	/* Should be enough */
 
-	strcpy(p, msg1);
-	strcat(p, disc);
+	strcpy(p, disc);
 	strcat(p, msg2);
 	strcat(p, authaddr);
 	strcat(p, msg3);
@@ -470,7 +468,7 @@ static void acctout(const char *disc)
 
 static RETSIGTYPE bye(int signum)
 {
-	acctout("TIMEOUT");
+	acctout("ERR: TIMEOUT");
 	exit(0);
 #if	RETSIGTYPE != void
 	return (0);
@@ -500,7 +498,7 @@ int	c;
 			printf("+OK Bye-bye.\r\n");
 			fflush(stdout);
 			cleanup();
-			acctout("LOGOUT");
+			acctout("INFO: LOGOUT");
 			return;
 		}
 
@@ -591,7 +589,7 @@ error:
 		printf("-ERR Invalid command.\r\n");
 		fflush(stdout);
 	}
-	acctout("DISCONNECTED");
+	acctout("ERR: DISCONNECTED");
 }
 
 /* Like every good Maildir reader, we purge the tmp subdirectory */
