@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.
+** Copyright 1998 - 2000 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -10,11 +10,13 @@
 #include	<stdlib.h>
 #include	<string.h>
 #include	<ctype.h>
+#include	<errno.h>
+#include	"unicode/unicode.h"
 #include	"searchinfo.h"
 #include	"imapwrite.h"
 #include	"imaptoken.h"
 
-static const char rcsid[]="$Id: searchinfo.c,v 1.5 2000/09/02 21:52:25 mrsam Exp $";
+static const char rcsid[]="$Id: searchinfo.c,v 1.6 2000/09/29 04:38:47 mrsam Exp $";
 
 
 static struct searchinfo *alloc_search(
@@ -38,6 +40,7 @@ struct searchinfo *p;
 		p=si->next;
 		if (si->as)	free(si->as);
 		if (si->bs)	free(si->bs);
+		if (si->cs)	free(si->cs);
 
 		if (si->sei.r)	free(si->sei.r);
 		free(si);
@@ -120,14 +123,14 @@ struct imaptoken *t=currenttoken();
 
 /* Additional initalization for search keys that will search msg contents */
 
-static void init_search(struct searchinfo *si, char *s)
+static void init_search(struct searchinfo *si)
 {
-unsigned i, j, *r;
+	unsigned i, j, *r;
+	const char *s=si->sei.string=si->bs;
 
-	for (i=0; s[i]; i++)
-		s[i]=toupper((int)(unsigned char)s[i]);
+	if (si->sei.r)
+		free(si->sei.r);
 
-	si->sei.string=s;
 	si->sei.i=0;
 	if ((si->sei.r=r=(unsigned *)malloc(sizeof(unsigned)*strlen(s))) == 0)
 		write_error_exit(0);
@@ -191,22 +194,20 @@ const char *keyword;
 	{
 	struct imaptoken *t;
 	struct searchinfo *si;
-	char	*p;
 
 		si=alloc_search(head);
 		si->type=search_header;
 		t=nexttoken();
 		if (t->tokentype != IT_ATOM && t->tokentype != IT_QUOTED_STRING)
 			return (0);
-		si->as=my_strdup(t->tokenbuf);
-		for (p=si->as; *p; p++)
-			*p=toupper((int)(unsigned char)*p);
+		si->cs=(*unicode_ISO8859_1.toupper_func)(t->tokenbuf, 0);
+		if (!si->cs)
+			write_error_exit(0);
 		t=nexttoken();
 		if (t->tokentype != IT_ATOM && t->tokentype != IT_QUOTED_STRING)
 			return (0);
-		si->bs=my_strdup(t->tokenbuf);
+		si->as=my_strdup(t->tokenbuf);
 		nexttoken();
-		init_search(si, si->bs);
 		return (si);
 	}
 
@@ -221,13 +222,12 @@ const char *keyword;
 
 		si=alloc_search(head);
 		si->type=search_header;
-		si->as=my_strdup(keyword);
+		si->cs=my_strdup(keyword);
 		t=nexttoken();
 		if (t->tokentype != IT_ATOM && t->tokentype != IT_QUOTED_STRING)
 			return (0);
-		si->bs=my_strdup(t->tokenbuf);
+		si->as=my_strdup(t->tokenbuf);
 		nexttoken();
-		init_search(si, si->bs);
 		return (si);
 	}
 
@@ -258,7 +258,6 @@ const char *keyword;
 			return (0);
 		si->as=my_strdup(t->tokenbuf);
 		nexttoken();
-		init_search(si, si->as);
 		return (si);
 	}
 	if (strcmp(keyword, "LARGER") == 0)
@@ -378,7 +377,6 @@ const char *keyword;
 			return (0);
 		si->as=my_strdup(t->tokenbuf);
 		nexttoken();
-		init_search(si, si->as);
 		return (si);
 	}
 
@@ -507,7 +505,55 @@ const char *keyword;
 	return (0);
 }
 
-#if 1
+/*
+** We are about to search in charset 'textcharset'.  Make sure that all
+** search_text nodes in the search string are in that character set.
+*/
+
+void search_set_charset_conv(struct searchinfo *si,
+			     const struct unicode_info *mycharset,
+			     const struct unicode_info *textcharset)
+{
+	for (; si; si=si->next)
+	{
+		char *p;
+
+		if (si->type != search_text && si->type != search_body
+		    && si->type != search_header)
+			continue;
+		if (si->value > 0)
+			continue; /* Already found, no need to do this again */
+
+		if (si->bs_charset &&
+		    strcmp(si->bs_charset->chset, textcharset->chset) == 0)
+			continue;	/* Already converted to this charset */
+
+		si->bs_charset=0;
+		if (si->bs)	free(si->bs);
+		si->bs=0;
+
+		p=unicode_convert(si->as ? si->as:"",
+				       mycharset,
+				       textcharset);
+
+		if (!p && errno != EINVAL)
+			write_error_exit(0);
+
+		if (p)
+		{
+			si->bs=(*mycharset->toupper_func)(p, 0);
+			free(p);
+			if (!si->bs)
+				write_error_exit(0);
+			init_search(si);
+		}
+
+		si->bs_charset=textcharset;
+	}
+}
+
+
+#if 0
 
 void debug_search(struct searchinfo *si)
 {
@@ -547,7 +593,7 @@ void debug_search(struct searchinfo *si)
 		return;
 	case search_header:
 		writes("HEADER \"");
-		writeqs(si->as);
+		writeqs(si->cs);
 		writes("\" \"");
 		writeqs(si->bs);
 		writes("\"");
