@@ -6,7 +6,7 @@
 ** See COPYING for distribution information.
 */
 
-static const char maildirfilter_h_rcsid[]="$Id: maildirfilter.h,v 1.4 2000/12/02 05:38:53 mrsam Exp $";
+static const char maildirfilter_h_rcsid[]="$Id: maildirfilter.h,v 1.6 2001/05/16 03:03:38 mrsam Exp $";
 
 #include	"config.h"
 
@@ -39,6 +39,7 @@ struct maildirfilterrule {
 	char *fieldname;
 	char *fieldvalue;
 	char *tofolder;
+	char *fromhdr;
 	} ;
 
 struct maildirfilter {
@@ -53,13 +54,16 @@ struct maildirfilter {
 ** A maildirfilter structure is initialized simply by nulling it out, then: */
 
 struct maildirfilterrule *mf_appendrule(struct maildirfilter *,
-	const char *,
-	enum maildirfiltertype,
-	int,
-	const char *,
-	const char *,
-	const char *,
-	int *);		/* Append a new rule */
+					const char *,
+					enum maildirfiltertype,
+					int,
+					const char *,
+					const char *,
+					const char *,
+					const char *,
+					int *);		/* Append a new rule */
+
+int mf_setautoreplyfrom(struct maildirfilter *, const char *);
 
 /* Move a given rule up or down in the hierarchy */
 
@@ -73,13 +77,14 @@ void mf_ruledel(struct maildirfilter *, struct maildirfilterrule *);
 /* Update an existing rule */
 
 int mf_ruleupdate(struct maildirfilter *, struct maildirfilterrule *,
-	const char *,
-        enum maildirfiltertype,
-	int,
-        const char *,
-        const char *,
-        const char *,
-	int *);
+		  const char *,
+		  enum maildirfiltertype,
+		  int,
+		  const char *,
+		  const char *,
+		  const char *,
+		  const char *,
+		  int *);
 
 #define	mf_freerules(r)	do { \
 	while ( (r)->first )	\
@@ -97,6 +102,7 @@ int mf_ruleupdate(struct maildirfilter *, struct maildirfilterrule *,
 #define	MF_ERR_BADRULEHEADER	3
 #define	MF_ERR_BADRULEVALUE	4
 #define	MF_ERR_BADRULEFOLDER	5
+#define MF_ERR_BADFROMHDR	6
 #define MF_ERR_INTERNAL		100
 
 /* Save/Load rules from the given file */
@@ -127,6 +133,21 @@ int mf_hasmaildirfilter(const char *);		/* Is maildir filter defined? */
 
 void mf_endmaildirfilter(const char *);		/* Remove the temp file */
 
+	/*
+	** A convenient structure to parse autoresponder parameters.
+	*/
+
+struct mf_autoresp_info {
+	char *name;
+	int dsnflag;
+	unsigned days;
+} ;
+
+int mf_autoresp_info_init_str(struct mf_autoresp_info *, const char *);
+int mf_autoresp_info_init(struct mf_autoresp_info *, const char *);
+void mf_autoresp_info_free(struct mf_autoresp_info *);
+char *mf_autoresp_info_asstr(struct mf_autoresp_info *);
+
 #ifdef __cplusplus
 }
 #endif
