@@ -41,7 +41,7 @@
 #include	"maildircreate.h"
 #include	"maildirsharedrc.h"
 
-static const char rcsid[]="$Id: maildirshared.c,v 1.13 2000/08/02 05:07:44 mrsam Exp $";
+static const char rcsid[]="$Id: maildirshared.c,v 1.14 2001/04/05 03:27:11 mrsam Exp $";
 
 /* Prerequisited for shared folder support */
 
@@ -883,7 +883,39 @@ char	*buf=maildir_getlink(filename);
 
 	if (buf)
 	{
-		unlink(buf);
+		struct stat stat_buf;
+		int rc=unlink(buf);
+
+		/*
+		** If we FAILED to unlink the real message in the real
+		** sharable folder, but the message still exists, it means
+		** that we do not have the permission to do so, so do not
+		** purge this folder.  Instead, remove the T flag from
+		** this message.
+		*/
+
+		if (rc && stat(buf, &stat_buf) == 0)
+		{
+			char *cpy=strdup(filename);
+
+			if (cpy)
+			{
+				char *p=strrchr(cpy, ':');
+
+				if (p && strchr(p, '/') == 0 &&
+				    strncmp(p, ":2,", 3) == 0 &&
+				    (p=strchr(p, 'T')) != 0)
+				{
+					while ((*p=p[1]) != 0)
+						++p;
+					rename(filename, cpy);
+				}
+				free(cpy);
+			}
+						
+			free(buf);
+			return (0);
+		}
 		free(buf);
 	}
 	unlink(filename);
