@@ -16,7 +16,7 @@
 #include	<dlfcn.h>
 #endif
 
-static const char rcsid[]="$Id: soxwrap.c,v 1.5 2000/12/24 04:18:25 mrsam Exp $";
+static const char rcsid[]="$Id: soxwrap.c,v 1.6 2001/04/13 21:04:23 mrsam Exp $";
 
 #if	SOX_DYNAMIC
 
@@ -53,7 +53,7 @@ static int fake_sox_init(char *p)
 
 struct funcptr init_ptr = {sox_n_init, (void *)&fake_sox_init};
 
-#define	DEF(n)	struct funcptr init_ ## n = { sox_n_ ##n, &n };
+#define	DEF(n)	struct funcptr init_ ## n = { sox_n_ ##n, (void *)&n };
 
 DEF(getpeername)
 DEF(getsockname)
