@@ -23,6 +23,7 @@
 
 
 #include	<sys/types.h>
+#include	<sys/time.h>
 #include	<arpa/inet.h>
 #include	<pwd.h>
 #include	<grp.h>
@@ -55,7 +56,7 @@
 
 #include	<netdb.h>
 
-static const char rcsid[]="$Id: tcpd.c,v 1.21 2001/01/24 05:30:14 mrsam Exp $";
+static const char rcsid[]="$Id: tcpd.c,v 1.23 2001/06/13 21:42:44 mrsam Exp $";
 
 static const char *accessarg=0;
 static const char *listenarg=0;
@@ -101,7 +102,16 @@ static struct args arginfo[]={
 	{0}
 	} ;
 
-static int fd;
+/* Ports we're listening on: */
+
+static struct portinfo {
+	struct portinfo *next;
+	const char *ipaddr;	/* Specific IP addr, or 0 */
+	const char *servname;	/* Service name/port */
+
+	int fd1, fd2;		/* BSD may need both IPv4 and IPv6 sockets */
+} *fdlist=0;
+static int maxfd;
 
 static int nprocs, maxperc, maxperip;
 static pid_t *pids;
@@ -185,28 +195,302 @@ static RETSIGTYPE sighup(int n)
 #endif
 }
 
+/*
+** Initialize a single listening socket
+*/
+
+static struct portinfo *createport(const char *a, const char *s)
+{
+	struct portinfo *p=(struct portinfo *)malloc(sizeof(struct portinfo));
+
+	if (!p)
+	{
+		perror("malloc");
+		return (NULL);
+	}
+
+	p->next=fdlist;
+	fdlist=p;
+	p->ipaddr=a;
+	p->servname=s;
+	p->fd1=p->fd2= -1;
+	return (p);
+}
+
+static int parseaddr(const char *p)
+{
+	char *buf=strdup(p);
+	char *q, *a, *s;
+
+	if (!p)
+	{
+		perror("malloc");
+		return (-1);
+	}
+
+	for (q=buf; (q=strtok(q, ",")) != NULL; q=0)
+	{
+		if ((s=strrchr(q, '.')) != 0)
+		{
+			*s++=0;
+			a=q;
+		}
+		else
+		{
+			a=0;
+			s=q;
+		}
+
+		if (createport(a, s) == NULL)
+			return (-1);
+	}
+
+	if (ipaddrarg)
+	{
+		struct portinfo *p;
+
+		for (p=fdlist; p; p=p->next)
+		{
+			if (p->ipaddr && strcmp(p->ipaddr, "0"))
+				continue;
+			p->ipaddr=ipaddrarg;
+		}
+	}
+
+	return (0);
+}
+
+/*
+** Create one socket, bound to a specific host/port
+*/
+
+static int mksocket(const char *ipaddrarg,	/* Host/IP address */
+		    const char *servname,	/* Service/port */
+		    int flags)
+
+#define MKS_USEAFINET4 1
+#define MKS_ERROK 2
+
+{
+	struct servent *servptr;
+	int	port;
+	int	fd;
+
+	RFC1035_ADDR addr;
+	RFC1035_NETADDR  netaddr;
+	const struct sockaddr *sinaddr;
+	int	sinaddrlen;
+
+#if	RFC1035_IPV6
+	struct sockaddr_in6	sin6;
+#endif
+
+	struct sockaddr_in	sin4;
+
+	int	af;
+
+	servptr=getservbyname(servname, "tcp");
+	if (servptr)
+		port=servptr->s_port;
+	else
+	{
+		port=atoi(servname);
+		if (port <= 0 || port > 65535)
+		{
+			fprintf(stderr, "Invalid port: %s\n", servname);
+			return (-1);
+		}
+		port=htons(port);
+	}
+
+	/* Create an IPv6 or an IPv4 socket */
+
+#if	RFC1035_IPV6
+	if (flags & MKS_USEAFINET4)
+	{
+		fd=socket(PF_INET, SOCK_STREAM, 0);
+		af=AF_INET;
+	}
+	else
+#endif
+		fd=rfc1035_mksocket(SOCK_STREAM, 0, &af);
+
+	if (fd < 0)
+	{
+		perror("socket");
+		return (-1);
+	}
+
+	/* Figure out what to bind based on what socket we created */
+
+	if (ipaddrarg && strcmp(ipaddrarg, "0"))
+	{
+		if (rfc1035_aton(ipaddrarg, &addr) < 0)
+		{
+			fprintf(stderr,"Invalid IP address: %s\n", ipaddrarg);
+			close(fd);
+			return (-1);
+		}
+
+		if (rfc1035_mkaddress(af, &netaddr, &addr, port, &sinaddr,
+				&sinaddrlen))
+		{
+			fprintf(stderr,"Unable to bind IP address: %s\n",
+				ipaddrarg);
+			close(fd);
+			return (-1);
+		}
+	}
+	else	/* Bind default address */
+	{
+#if	RFC1035_IPV6
+		if (af == AF_INET6)
+		{
+			memset(&sin6, 0, sizeof(sin6));
+			sin6.sin6_family=AF_INET6;
+			sin6.sin6_addr=in6addr_any;
+			sin6.sin6_port=port;
+			sinaddr=(const struct sockaddr *)&sin6;
+			sinaddrlen=sizeof(sin6);
+		}
+		else
+#endif
+			if (af == AF_INET)
+			{
+				sin4.sin_family=AF_INET;
+				sin4.sin_addr.s_addr=INADDR_ANY;
+				sin4.sin_port=port;
+				sinaddr=(const struct sockaddr *)&sin4;
+				sinaddrlen=sizeof(sin4);
+			}
+			else
+			{
+				errno=EAFNOSUPPORT;
+				perror("socket");
+				close(fd);
+				return (-1);
+			}
+	}
+
+	{
+		int dummy=1;
+
+		if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
+		       (const char *)&dummy, sizeof(dummy)) < 0)
+		{
+			perror("setsockopt");
+		}
+	}
+
+	if (fcntl(fd, F_SETFD, FD_CLOEXEC))
+	{
+		perror("fcntl");
+		close(fd);
+		return (-1);
+	}
+
+	if (fcntl(fd, F_SETFL, O_NONBLOCK))
+	{
+		perror("fcntl");
+		close(fd);
+		return (-1);
+	}
+
+	if (sox_bind(fd, (struct sockaddr *)sinaddr, sinaddrlen) < 0)
+	{
+		if (flags && MKS_ERROK)
+		{
+			close(fd);
+			return (-2);
+		}
+
+		perror("bind");
+		close(fd);
+		return (-1);
+	}
+
+	if (sox_listen(fd,
+#ifdef	SOMAXCONN
+		       SOMAXCONN
+#else
+		       5
+#endif
+		       ))
+	{
+		if (flags && MKS_ERROK)
+		{
+			close(fd);
+			return (-2);
+		}
+		perror("listen");
+		close(fd);
+		return (-1);
+	}
+	return (fd);
+}
+
+static int mksockets()
+{
+	struct portinfo *p;
+
+	maxfd= -1;
+
+	for (p=fdlist; p; p=p->next)
+	{
+		int fd=mksocket(p->ipaddr, p->servname, 0);
+
+		if (fd < 0)
+			break;
+
+		p->fd1=fd;
+
+		if (fd > maxfd)
+			maxfd=fd;
+
+		/* BSD requires both an IPv6 and an IPv4 socket */
+
+#if	RFC1035_IPV6
+		if (p->ipaddr == 0 || strcmp(p->ipaddr, "0") == 0)
+		{
+			fd=mksocket(p->ipaddr, p->servname,
+				    (MKS_USEAFINET4|MKS_ERROK));
+
+			if (fd == -2)
+				continue;	/* Ok if bind failed */
+			if (fd < 0)
+				break;
+
+			if (fd > maxfd)
+				maxfd=fd;
+			p->fd2=fd;
+		}
+#endif
+
+	}
+
+	if (p)	/* Clean up after ourselves, after an error */
+	{
+		for (p=fdlist; p; p=p->next)
+		{
+			if (p->fd1 >= 0)
+				close(p->fd1);
+			if (p->fd2 >= 0)
+				close(p->fd2);
+		}
+		return (-1);
+	}
+
+	return (0);
+}
+
 static int init(int argc, char **argv)
 {
 int	argn;
-int	port;
 
-RFC1035_ADDR addr;
-RFC1035_NETADDR  netaddr;
-const struct sockaddr *sinaddr;
-int	sinaddrlen;
-
-#if	RFC1035_IPV6
-struct sockaddr_in6	sin6;
-#endif
-
-struct sockaddr_in	sin4;
-
-int	af;
 struct	group *gr;
 int	i;
 gid_t	gid=0;
 const	char *servname;
-struct servent *servptr;
 int	forced=0;
 int	lockfd;
  
@@ -254,99 +538,14 @@ int	lockfd;
 
 	servname=argv[argn++];
 
-	servptr=getservbyname(servname, "tcp");
-	if (servptr)
-		port=servptr->s_port;
-	else
+	if (parseaddr(servname))
 	{
-		port=atoi(servname);
-		if (port <= 0 || port > 65535)
-		{
-			fprintf(stderr, "Invalid port: %s\n", servname);
-			return (-1);
-		}
-		port=htons(port);
-	}
-
-	/* Create an IPv6 or an IPv4 socket */
-
-	if ((fd=rfc1035_mksocket(SOCK_STREAM, 0, &af)) < 0)
-	{
-		perror("socket");
 		close(lockfd);
 		return (-1);
 	}
 
-	/* Figure out what to bind based on what socket we created */
-
-	if (ipaddrarg && strcmp(ipaddrarg, "0"))
+	if (mksockets())
 	{
-		if (rfc1035_aton(ipaddrarg, &addr) < 0)
-		{
-			fprintf(stderr,"Invalid IP address: %s\n", ipaddrarg);
-			close(fd);
-			close(lockfd);
-			return (-1);
-		}
-
-		if (rfc1035_mkaddress(af, &netaddr, &addr, port, &sinaddr,
-				&sinaddrlen))
-		{
-			fprintf(stderr,"Unable to bind IP address: %s\n",
-				ipaddrarg);
-			close(fd);
-			close(lockfd);
-			return (-1);
-		}
-	}
-	else	/* Bind default address */
-	{
-#if	RFC1035_IPV6
-
-		if (af == AF_INET6)
-		{
-			memset(&sin6, 0, sizeof(sin6));
-			sin6.sin6_family=AF_INET6;
-			sin6.sin6_addr=in6addr_any;
-			sin6.sin6_port=port;
-			sinaddr=(const struct sockaddr *)&sin6;
-			sinaddrlen=sizeof(sin6);
-		}
-		else
-#endif
-		if (af == AF_INET)
-		{
-			sin4.sin_family=AF_INET;
-			sin4.sin_addr.s_addr=INADDR_ANY;
-			sin4.sin_port=port;
-			sinaddr=(const struct sockaddr *)&sin4;
-			sinaddrlen=sizeof(sin4);
-		}
-		else
-		{
-			errno=EAFNOSUPPORT;
-			perror("socket");
-			close(fd);
-			close(lockfd);
-			return (-1);
-		}
-	}
-
-	{
-		int dummy=1;
-
-		if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
-		       (const char *)&dummy, sizeof(dummy)) < 0)
-		{
-			perror("setsockopt");
-		}
-	}
-
-
-	if (sox_bind(fd, (struct sockaddr *)sinaddr, sinaddrlen) < 0)
-	{
-		perror("bind");
-		close(fd);
 		close(lockfd);
 		return (-1);
 	}
@@ -400,7 +599,6 @@ int	lockfd;
 		else if ((gr=getgrnam(grouparg)) == 0)
 		{
 			fprintf(stderr, "Group not found: %s\n", grouparg);
-			sox_close(fd);
 			close(lockfd);
 			return (-1);
 		}
@@ -430,7 +628,6 @@ int	lockfd;
 	if (ll_daemon_resetio())
 	{
 		perror("ll_daemon_resetio");
-		close(fd);
 		close(lockfd);
 		return (-1);
 	}
@@ -465,7 +662,6 @@ int	lockfd;
 			open("/dev/null", O_WRONLY);
 			sox_close(2);
 			sox_dup(1);
-			close(fd);
 			closeaccess();
 			while ((p=fork()) == -1)
 			{
@@ -516,7 +712,6 @@ int	lockfd;
 		if (nprocs <= 0)
 		{
 			fprintf(stderr, "Invalid -maxprocsarg option.\n");
-			sox_close(fd);
 			return (-1);
 		}
 	}
@@ -524,14 +719,12 @@ int	lockfd;
 	if ((pids=malloc(sizeof(*pids)*nprocs)) == 0)
 	{
 		perror("malloc");
-		sox_close(fd);
 		return (-1);
 	}
 	if ((addrs=malloc(sizeof(*addrs)*nprocs)) == 0)
 	{
 		free(pids);
 		perror("malloc");
-		sox_close(fd);
 		return (-1);
 	}
 
@@ -550,7 +743,6 @@ int	lockfd;
 			fprintf(stderr, "Invalid -maxperc option.\n");
 			free(pids);
 			free(addrs);
-			sox_close(fd);
 			return (-1);
 		}
 	}
@@ -562,7 +754,6 @@ int	lockfd;
 			fprintf(stderr, "Invalid -maxperip option.\n");
 			free(pids);
 			free(addrs);
-			sox_close(fd);
 			return (-1);
 		}
 	}
@@ -720,11 +911,41 @@ int	l;
 	return (1);
 }
 
+/* Wait until we have at least one available slot left */
+
+static int getfreeslot(int *pidptr)
+{
+	int n;
+
+	for (;;)
+	{
+		wait_block();
+
+		for (n=0; n<nprocs; n++)
+		{
+			if (pids[*pidptr] == (pid_t)-1)	break;
+			if (++*pidptr >= nprocs)	*pidptr=0;
+		}
+		if (pids[*pidptr] != (pid_t)-1)
+		{
+			wait_forchild(doreap, childsig);
+			continue;
+		}
+		break;
+	}
+	wait_clear(childsig);
+	return (*pidptr);
+}
+
+static void accepted(int, int, RFC1035_NETADDR *, int, const char *, char **);
+
 static int doit(int argn, int argc, char **argv)
 {
-char	**ptrs;
-int	dummy;
-int	pidptr;
+	char	**ptrs;
+	int	pidptr;
+	struct portinfo *pi;
+	fd_set fdr, fdrcopy;
+	int	dummy;
 
 	ptrs=(char **)malloc((argc-argn+1) * sizeof(char *));
 	if (!ptrs)
@@ -738,12 +959,6 @@ int	pidptr;
 	}
 	ptrs[dummy]=0;
 
-#ifdef	SOMAXCONN
-	dummy=SOMAXCONN;
-#else
-	dummy=5
-#endif
-
 	if (listenarg)
 	{
 		dummy=atoi(listenarg);
@@ -754,10 +969,14 @@ int	pidptr;
 		}
 	}
 
-	if (sox_listen(fd, dummy))
+	FD_ZERO(&fdrcopy);
+	for (pi=fdlist; pi; pi=pi->next)
 	{
-		perror("listen");
-		return (-1);
+		if (pi->fd1 >= 0)
+			FD_SET(pi->fd1, &fdrcopy);
+
+		if (pi->fd2 >= 0)
+			FD_SET(pi->fd2, &fdrcopy);
 	}
 
 	pidptr=0;
@@ -790,103 +1009,137 @@ int	pidptr;
 
 	for (;;)
 	{
-	int	sockfd;
-	RFC1035_NETADDR	sin;
-	RFC1035_ADDR addr;
-	int	addrport;
-	int	sinl;
-#ifdef	SO_LINGER
-	struct	linger l;
-#endif
-	pid_t	p;
-	int	n;
+		int n;
+		int sockfd;
+		RFC1035_NETADDR	sin;
+		int	sinl;
 
-		wait_block();
+		fdr=fdrcopy;
 
-		for (n=0; n<nprocs; n++)
-		{
-			if (pids[pidptr] == (pid_t)-1)	break;
-			if (++pidptr >= nprocs)	pidptr=0;
-		}
-		if (pids[pidptr] != (pid_t)-1)
-		{
-			wait_forchild(doreap, childsig);
-			continue;
-		}
-		wait_clear(childsig);
-
-		sinl=sizeof(sin);
-
-		while ((sockfd=sox_accept(fd, (struct sockaddr *)&sin, &sinl)) < 0)
+		if (select(maxfd+1, &fdr, NULL, NULL, NULL) <= 0)
 		{
 			if (errno != EINTR)
 				perror("accept");
-		}
-
-		if (rfc1035_sockaddrip(&sin, sinl, &addr)
-			|| rfc1035_sockaddrport(&sin, sinl, &addrport))
-		{
-			sox_close(sockfd);
 			continue;
 		}
 
-		if (sighup_received)
+		for (pi=fdlist; pi; pi=pi->next)
 		{
-			sighup_received=0;
-			if (accessarg)
+			if (pi->fd1 >= 0 && FD_ISSET(pi->fd1, &fdr) &&
+			    ((n=getfreeslot(&pidptr)),
+			     (sinl = sizeof(struct sockaddr)),
+			     (sockfd=sox_accept(pi->fd1,
+						(struct sockaddr *)&sin,
+						&sinl))) >= 0)
 			{
-				closeaccess();
-				if (openaccess(accessarg))
-					perror(accessarg);
+				accepted(n, sockfd, &sin, sinl,
+					 argv[argn], ptrs);
+			}
+
+			if (pi->fd2 >= 0 && FD_ISSET(pi->fd2, &fdr) &&
+			    ((n=getfreeslot(&pidptr)),
+			     (sinl = sizeof(struct sockaddr)), 
+			     (sockfd=sox_accept(pi->fd2,
+						(struct sockaddr *)&sin,
+						&sinl))) >= 0)
+			{
+				accepted(n, sockfd, &sin, sinl,
+					 argv[argn], ptrs);
 			}
 		}
+	}
+}
 
-		dummy=1;
-#ifdef	SO_KEEPALIVE
-		if (setsockopt(sockfd, SOL_SOCKET, SO_KEEPALIVE,
-			(const char *)&dummy, sizeof(dummy)) < 0)
+static void accepted(int n, int sockfd, RFC1035_NETADDR *sin, int sinl,
+		     const char *prog,
+		     char **args)
+{
+	RFC1035_ADDR addr;
+	int	addrport;
+#ifdef	SO_LINGER
+	int	dummy;
+	struct	linger l;
+#endif
+	pid_t	p;
+
+	if (rfc1035_sockaddrip(sin, sinl, &addr)
+	    || rfc1035_sockaddrport(sin, sinl, &addrport))
+	{
+		sox_close(sockfd);
+		return;
+	}
+
+	/* Turn off the CLOEXEC and NONBLOCK bits */
+
+	if (fcntl(sockfd, F_SETFD, 0))
+	{
+		perror("fcntl");
+		sox_close(sockfd);
+		return;
+	}
+
+	if (fcntl(sockfd, F_SETFL, 0))
+	{
+		perror("fcntl");
+		sox_close(sockfd);
+		return;
+	}
+
+	if (sighup_received)
+	{
+		sighup_received=0;
+		if (accessarg)
 		{
-			perror("setsockopt");
+			closeaccess();
+			if (openaccess(accessarg))
+				perror(accessarg);
 		}
+	}
+
+#ifdef	SO_KEEPALIVE
+	dummy=1;
+	if (setsockopt(sockfd, SOL_SOCKET, SO_KEEPALIVE,
+		       (const char *)&dummy, sizeof(dummy)) < 0)
+	{
+		perror("setsockopt");
+	}
 #endif
 
 #ifdef	SO_LINGER
-		l.l_onoff=0;
-		l.l_linger=0;
+	l.l_onoff=0;
+	l.l_linger=0;
 
-		if (setsockopt(sockfd, SOL_SOCKET, SO_LINGER,
-			(const char *)&l, sizeof(l)) < 0)
-		{
-			perror("setsockopt");
-		}
-#endif
-		wait_block();
-		if ((p=fork()) == -1)
-		{
-			perror("fork");
-			sox_close(sockfd);
-			continue;
-		}
-
-		if (p == 0)
-		{
-			wait_restore(childsig);
-			sox_close(fd);
-
-			if (allowaccess(&addr) == 0)
-			{
-				sox_close(sockfd);
-				_exit(0);
-			}
-
-			run(sockfd, &addr, addrport, argv[argn], ptrs);
-		}
-		pids[pidptr]=p;
-
-		memcpy(addrs+pidptr, &addr, sizeof(addr));
-		sox_close(sockfd);
-		wait_clear(childsig);
+	if (setsockopt(sockfd, SOL_SOCKET, SO_LINGER,
+		       (const char *)&l, sizeof(l)) < 0)
+	{
+		perror("setsockopt");
 	}
+#endif
+	wait_block();
+	if ((p=fork()) == -1)
+	{
+		perror("fork");
+		sox_close(sockfd);
+		return;
+	}
+
+	if (p == 0)
+	{
+		wait_restore(childsig);
+
+		if (allowaccess(&addr) == 0)
+		{
+			sox_close(sockfd);
+			_exit(0);
+		}
+
+		run(sockfd, &addr, addrport, prog, args);
+	}
+	pids[n]=p;
+
+	memcpy(addrs+n, &addr, sizeof(addr));
+	sox_close(sockfd);
+	wait_clear(childsig);
 }
 
 static void mysetenv(const char *name, const char *val)
